﻿#ifndef ELECTRON_PLUGINS_ITRTCPLUGIN_
#define ELECTRON_PLUGINS_ITRTCPLUGIN_

#if defined(_WIN32)
#define EXPORTS  __declspec(dllexport)
#else
#define EXPORTS  __attribute__((visibility("default")))
#endif

typedef void (*PluginErrorCallbackFunc)(int error, const char* msg, void* param);

enum class TRTC_PLUGIN_TYPE:int {
    TRTC_PLUGIN_UNKNOWN = 0,
    TRTC_PLUGIN_VIDEO = 1,
    TRTC_PLUGIN_MEDIAENCRYPTDECRYPT = 2,
};

class ITRTCPlugin {
public:
    virtual ~ITRTCPlugin() {}

    /**
     * 初始化插件
     *
     * 创建插件后，会立刻触发该初始化函数，用户可以在这里做一些初始化操作。
     * @return true: 插件初始化成功，false: 插件初始化失败。
     *   如果返回 false 表示初始化失败，Javascript 层通过 setPluginCallback() 函数设置回调函数会收到错误码：-4.
     */
    virtual bool init() = 0;

    /**
     * 反初始化插件
     *
     * 插件销毁时，会触发该函数，用户可以在这里做一些清理操作。
     *
     * @return true: 插件反初始化成功，false: 插件反初始化失败。
     */
    virtual bool uninit() = 0;

    /**
     * 资源加载
     *
     * 会在 init() 函数之后被调用。
     *
     * @param path 插件资源目录。该目录为当前插件库文件所在目录，如果插件有其它资源文件依赖，可以放在插件库文件同目录下，在这里加载。
     * @return true: 资源加载成功，false: 资源加载失败。
     *   如果返回 false 表示资源加载失败，Javascript 层通过 setPluginCallback() 函数设置回调函数会收到错误码：-5.
     */
    virtual bool load(const char* path) = 0;

    /**
     * 资源卸载
     *
     * 会在 uninit() 函数之前被调用。
     */
    virtual bool unload() = 0;

    /**
     * 开启/关闭当前插件
     *
     * Javascript 层的 TRTCPluginInfo.enable/disable() 接口调用会触发该函数。
     *
     * @param enabled true: 开启插件，false: 关闭插件。
     * @return true: 成功开启/关闭插件，false: 失败。
     */
    virtual bool enable(bool enabled) = 0;

    /**
     * 设置插件参数
     *
     * Javascript 层的 TRTCPluginInfo.setParameter() 接口调用会触发该函数。
     *
     * @param param 插件参数，格式为 json 字符串。
     * @return true: 设置成功，false: 设置失败。
     */
    virtual bool setParameter(const char* param) = 0;

    /**
     * 获取插件类型
     *
     * 具体的插件子类中需要重写。
     */
    virtual TRTC_PLUGIN_TYPE getPluginType() {
        return TRTC_PLUGIN_TYPE::TRTC_PLUGIN_UNKNOWN;
    }

    /**
     * 设置错误回调函数
     *
     * @param callback 回调函数。是对 Javascript 层 setPluginCallback() 接口设置回调函数的封装。
     */
    void setErrorCallback(PluginErrorCallbackFunc callback, void* param) {
        error_callback_ = callback;
        param_ = param;
    }

    /**
     * 错误码上报函数
     *
     * 该函数调用后，会触发 Javascript 层 setPluginCallback() 接口设置的回调函数，将错误码和错误信息透传给 Javascript 层，用户可以根据错误码做相应处理。
     *
     * @param error 错误码。
     * @param msg 错误描述。
     */
    void onError(int error, const char* msg) {
        if (error_callback_) {
            error_callback_(error, msg, param_);
        }
    }

  private:
      PluginErrorCallbackFunc error_callback_ = nullptr;
      void* param_ = nullptr;
};

// 创建插件实例对象，Javascript 层 addPlugin() 接口最终会调用到该函数
extern "C" EXPORTS ITRTCPlugin* createTRTCPlugin();

// 销毁插件实例对象，Javascript 层 removePlugin() 接口最终会调用到该函数
extern "C" EXPORTS void destoryTRTCPlugin(ITRTCPlugin*);

#endif  // ELECTRON_PLUGINS_ITRTCPLUGIN_
